import { createReducer, on } from '@ngrx/store';
import { hashKey, isCacheValid, } from './cache.models';
import * as CacheActions from './cache.actions';
export const initialState = {
    keys: {},
};
export const cacheReducer = createReducer(initialState, on(CacheActions.cache, (state, { key, value, date, maxCacheSize }) => setCacheValue(hashKey(key), { value, date, invalid: false }, state, maxCacheSize)), on(CacheActions.invalidateCache, (state, { key }) => {
    const k = hashKey(key);
    return invalidateCache(k, state);
}), on(CacheActions.deleteCache, (state, { key }) => {
    const k = hashKey(key);
    return deleteCacheValue(k, state);
}), on(CacheActions.hitCache, (state, { key }) => {
    const k = hashKey(key);
    return increaseCacheHitCount(k, state);
}));
function setCacheValue(keys, value, state, maxCacheSize, expires) {
    const newState = Object.assign({}, state);
    let cache = newState;
    let lastCache = undefined;
    for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        cache.keys = (cache === null || cache === void 0 ? void 0 : cache.keys) ? Object.assign({}, cache === null || cache === void 0 ? void 0 : cache.keys) : {};
        let v = cache.keys[key];
        v = v ? Object.assign({}, v) : {};
        cache.keys[key] = v;
        lastCache = cache;
        cache = v;
    }
    cache.data = cache.data
        ? Object.assign(Object.assign({}, value), { hitCount: cache.data.hitCount + 1 }) : Object.assign(Object.assign({}, value), { hitCount: 1 });
    if (maxCacheSize &&
        (lastCache === null || lastCache === void 0 ? void 0 : lastCache.keys) &&
        Object.keys(lastCache.keys).length > maxCacheSize) {
        const entries = findLessHitOrOldestCacheEntries(lastCache, expires !== null && expires !== void 0 ? expires : Infinity, maxCacheSize);
        if (entries && entries.length) {
            for (const [key] of entries) {
                delete lastCache.keys[key];
            }
        }
    }
    return newState;
}
function findLessHitOrOldestCacheEntries(state, expires, maxCacheSize) {
    if (!state.keys)
        return undefined;
    const entries = Object.entries(state.keys);
    // find the newest key;
    const [newestKey] = entries.reduce((a, b) => {
        var _a, _b, _c, _d;
        const aDate = (_b = (_a = a[1].data) === null || _a === void 0 ? void 0 : _a.date) !== null && _b !== void 0 ? _b : 0;
        const bDate = (_d = (_c = b[1].data) === null || _c === void 0 ? void 0 : _c.date) !== null && _d !== void 0 ? _d : 0;
        return aDate > bDate ? a : b;
    });
    const sorted = entries.sort(([aKey, aValue], [bKey, bValue]) => {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        // ensures the newest key always wins
        if (aKey === newestKey)
            return -1;
        if (bKey === newestKey)
            return 1;
        const aValid = aValue.data && isCacheValid(aValue.data, expires) ? 1 : 0;
        const bValid = bValue.data && isCacheValid(bValue.data, expires) ? 1 : 0;
        const diffValid = aValid - bValid;
        const diffHit = ((_b = (_a = aValue.data) === null || _a === void 0 ? void 0 : _a.hitCount) !== null && _b !== void 0 ? _b : 0) - ((_d = (_c = bValue.data) === null || _c === void 0 ? void 0 : _c.hitCount) !== null && _d !== void 0 ? _d : 0);
        const diffDate = ((_f = (_e = aValue.data) === null || _e === void 0 ? void 0 : _e.date) !== null && _f !== void 0 ? _f : 0) - ((_h = (_g = bValue.data) === null || _g === void 0 ? void 0 : _g.date) !== null && _h !== void 0 ? _h : 0);
        return (-1 * (diffValid === 0 ? (diffHit === 0 ? diffDate : diffHit) : diffValid));
    });
    return sorted.slice(maxCacheSize);
}
function deleteCacheValue(keys, state) {
    const newState = Object.assign({}, state);
    let cache = newState;
    for (const key of keys) {
        if (!cache.keys)
            return state;
        cache.keys = Object.assign({}, cache === null || cache === void 0 ? void 0 : cache.keys);
        let v = cache.keys[key];
        if (!v)
            return state;
        v = Object.assign({}, v);
        cache.keys[key] = v;
        cache = v;
    }
    if (cache.data)
        delete cache.data;
    else if (cache.keys)
        delete cache.keys;
    return newState;
}
function invalidateCache(keys, state) {
    var _a;
    const newState = Object.assign({}, state);
    let cache = newState;
    for (const key of keys) {
        if (!(cache === null || cache === void 0 ? void 0 : cache.keys))
            return state;
        cache.keys = Object.assign({}, cache === null || cache === void 0 ? void 0 : cache.keys);
        let v = (_a = cache === null || cache === void 0 ? void 0 : cache.keys) === null || _a === void 0 ? void 0 : _a[key];
        if (!v)
            return state;
        v = Object.assign({}, v);
        cache.keys[key] = v;
        cache = v;
    }
    cache && invalidaSubKeys(cache);
    return newState;
}
function increaseCacheHitCount(keys, state) {
    var _a;
    const newState = Object.assign({}, state);
    let cache = newState;
    for (const key of keys) {
        if (!(cache === null || cache === void 0 ? void 0 : cache.keys))
            return state;
        cache.keys = Object.assign({}, cache === null || cache === void 0 ? void 0 : cache.keys);
        let v = (_a = cache === null || cache === void 0 ? void 0 : cache.keys) === null || _a === void 0 ? void 0 : _a[key];
        if (!v)
            return state;
        v = Object.assign({}, v);
        cache.keys[key] = v;
        cache = v;
    }
    if (!cache.data)
        return state;
    cache.data = Object.assign(Object.assign({}, cache.data), { hitCount: cache.data.hitCount + 1 });
    return newState;
}
function invalidaSubKeys(state) {
    if (state.data) {
        state.data = Object.assign(Object.assign({}, state.data), { invalid: true });
    }
    if (state.keys) {
        state.keys = Object.assign({}, state.keys);
        for (const key in state.keys) {
            state.keys[key] = invalidaSubKeys(Object.assign({}, state.keys[key]));
        }
    }
    return state;
}
//# sourceMappingURL=cache.reducer.js.map