import { of } from 'rxjs';
import { isCacheValid } from './cache.models';
import { selectCache } from './cache.selectors';
import { concatMap, first, tap } from 'rxjs/operators';
import * as CacheActions from './cache.actions';
export function cache({ store, key, source, expires, maxCacheSize, }) {
    const exp = expires !== null && expires !== void 0 ? expires : Infinity;
    return store.select(selectCache(key)).pipe(first(), concatMap((cache) => cache && isCacheValid(cache, exp)
        ? of(cache.value).pipe(tap(() => store.dispatch(CacheActions.hitCache({ key }))))
        : source.pipe(tap((value) => store.dispatch(CacheActions.cache({
            key,
            date: Date.now(),
            value,
            maxCacheSize,
        }))))));
}
//# sourceMappingURL=cache.service.js.map