import { SetEntityState } from './set-entity.model';
import { ActionCreatorProps } from '@ngrx/store/src/models';
declare type RecordEntity<T> = T extends Record<string, infer J> ? J : never;
/**
 * Generates ngrx code needed to set and entity in the store state
 * @param entityName - Entity name, should be in camel case
 * @param options.actionProps - param for the main request action,
 * use the props() function for its value
 * @returns the trait factory
 *
 * @example
 * const traits = createEntityFeatureFactory(
 * addSetEntityTraits({
 *        entityName: 'client',
 *        actionProps: props<{ client: { id: number; name: string } }
 *      }),
 * )({
 *      actionsGroupKey: 'Client',
 *      featureSelector: createFeatureSelector<
 *        SetEntityState<Client, 'client'>
 *        >('client'),
 *    });
 *
 * // will generate
 * traits.actions.setClient({client: {id:123, name: 'gabs'}});
 * traits.selectors.selectClient
 */
export declare function addSetEntityTrait<J extends string, Payload extends Record<J, any> | undefined = undefined, Entity = RecordEntity<Payload>, State = SetEntityState<Entity, J>>({ entityName, actionProps, }: {
    entityName: J;
    actionProps?: ActionCreatorProps<Payload>;
}): import("ngrx-traits").TraitFactory<State, import("ngrx-traits").PostfixProps<{
    set: import("@ngrx/store").ActionCreator<string, (props: Payload) => import("@ngrx/store/src/models").TypedAction<string>>;
}, J>, import("ngrx-traits").PostfixProps<{
    select: (state: SetEntityState<Entity, J>) => Entity;
}, J>, {}, string, {
    entityName: J;
    actionProps: ActionCreatorProps<Payload> | undefined;
}, import("ngrx-traits").AllTraitConfigs>;
export {};
